import { __assign } from "tslib";
import { fabric } from 'fabric-all-modules';
import { getCursor } from './cursors';
export var modes = {
    PENCIL: 'PENCIL',
    ERASERDRAW: 'ERASERDRAW',
    MOVE: 'MOVE'
};
var Board = /** @class */ (function () {
    function Board(params) {
        var _this = this;
        this.cursorPencil = getCursor('pencil');
        this.mouseDown = false;
        this.mouseRightDown = false;
        this.drawInstance = null;
        this.canvasConfig = {
            zoom: 1,
            contentJSON: null,
            minZoom: 0.05,
            maxZoom: 9.99,
            viewportTransform: [1, 0, 0, 1, 0, 0],
        };
        if (params) {
            this.drawingSettings = params.drawingSettings;
        }
        this.canvas = this.initCanvas(this.canvasConfig);
        this.canvas.once('after:render', function () {
            _this.applyCanvasConfig(params.canvasConfig);
        });
        this.modes = modes;
        this.resetZoom();
        this.setDrawingMode(this.drawingSettings.currentMode);
        this.addZoomListeners();
    }
    Board.prototype.initCanvas = function () {
        fabric.Canvas.prototype.getItemByAttr = function (attr, name) {
            var object = null, objects = this.getObjects();
            for (var i = 0, len = this.size(); i < len; i++) {
                if (objects[i][attr] && objects[i][attr] === name) {
                    object = objects[i];
                    break;
                }
            }
            return object;
        };
        var canvasElement = document.getElementById('canvas');
        if (!canvasElement)
            return;
        var parentElement = canvasElement.parentNode;
        var canvas = new fabric.Canvas('canvas', {
            fireRightClick: true,
            stopContextMenu: true, // 禁止默认右键菜单
        });
        canvas.perPixelTargetFind = true;
        if (parentElement) {
            var that = this;
            this.element = this.handleResize(this.resizeCanvas(canvas, parentElement).bind(this));
            this.element.observe(parentElement);
        }
        return canvas;
    };
    Board.prototype.applyCanvasConfig = function (canvasConfig) {
        this.canvasConfig = __assign(__assign({}, this.canvasConfig), canvasConfig);
        if (this.canvasConfig.zoom) {
            this.canvas.setZoom(this.canvasConfig.zoom);
        }
        if (this.canvasConfig.contentJSON) {
            this.canvas.loadFromJSON(this.canvasConfig.contentJSON);
        }
        if (this.canvasConfig.viewportTransform) {
            this.canvas.viewportTransform = this.canvasConfig.viewportTransform;
            this.changeZoom({ scale: 1 });
        }
        this.canvas.requestRenderAll();
        // console.log(this.canvas.getObjects());
        this.canvas.fire('config:change');
    };
    Board.prototype.addZoomListeners = function () {
        var canvas = this.canvas;
        var that = this;
        canvas.off('mouse:wheel');
        canvas.off('touch:gesture');
        canvas.on('mouse:wheel', function (opt) {
            opt.e.preventDefault();
            opt.e.stopPropagation();
            if (opt.e.shiftKey) {
                var delta = opt.e.deltaY;
                var scale = Math.pow(0.995, delta);
                var point = { x: opt.e.offsetX, y: opt.e.offsetY };
                that.changeZoom({ point: point, scale: scale });
            }
            else if (opt.e.ctrlKey) {
                var currWidth = that.drawingSettings.brushWidth;
                var delta = 1;
                if (opt.e.deltaY > 0) {
                    delta = -1;
                }
                var scale = currWidth + delta;
                if (0 < scale && scale <= 30) {
                    canvas.fire('brush:change', { target: { value: scale }, currentMode: that.drawingSettings.currentMode });
                }
            }
            else {
                // const e = opt.e;
                // let vpt = canvas.viewportTransform;
                // vpt[4] -= e.deltaX;
                // vpt[5] -= e.deltaY;
                // const boundaries = that.getCanvasContentBoundaries();
                // let scrolledX = vpt[4] + e.deltaX;
                // let scrolledY = vpt[5] + e.deltaY;
                // console.log('scrolled', scrolledX, scrolledY);
                // console.log('boundaries', boundaries);
                // const offset = 50;
                // scrolledX =
                //   scrolledX < -boundaries.maxX + offset
                //     ? -boundaries.maxX + offset
                //     : -scrolledX < boundaries.minX - canvas.width + offset
                //     ? canvas.width - boundaries.minX - offset
                //     : scrolledX;
                // scrolledY =
                //   scrolledY < -boundaries.maxY + offset
                //     ? -boundaries.maxY + offset
                //     : -scrolledY < boundaries.minY - canvas.height + offset
                //     ? canvas.height - boundaries.minY - offset
                //     : scrolledY;
                // that.throttle(() => console.log('after', scrolledX, scrolledY));
                // vpt[4] = scrolledX;
                // vpt[5] = scrolledY;
                // console.log(vpt);
                // canvas.requestRenderAll();
            }
        });
        canvas.on('touch:gesture', function (event) {
            // console.log('1 touch:gesture');
            if (event.e.touches && event.e.touches.length === 2) {
                var point1_1 = {
                    x: event.e.touches[0].clientX,
                    y: event.e.touches[0].clientY,
                };
                var point2_1 = {
                    x: event.e.touches[1].clientX,
                    y: event.e.touches[1].clientY,
                };
                var prevDistance_1 = canvas.getPointerDistance(point1_1, point2_1);
                canvas.on('touch:gesture', function (event) {
                    // console.log('2 touch:gesture');
                    var newDistance = canvas.getPointerDistance(point1_1, point2_1);
                    var zoom = newDistance / prevDistance_1;
                    var point = {
                        x: (point1_1.x + point2_1.x) / 2,
                        y: (point1_1.y + point2_1.y) / 2,
                    };
                    var scale = zoom;
                    that.changeZoom({ point: point, scale: scale });
                    canvas.renderAll();
                    prevDistance_1 = newDistance;
                });
            }
        });
    };
    Board.prototype.setDrawingSettings = function (drawingSettings) {
        if (!drawingSettings)
            return;
        this.drawingSettings = __assign(__assign({}, this.drawingSettings), drawingSettings);
        this.setDrawingMode(this.drawingSettings.currentMode);
    };
    Board.prototype.setCanvasConfig = function (canvasConfig) {
        if (!canvasConfig)
            return;
        this.applyCanvasConfig(canvasConfig);
    };
    Board.prototype.setDrawingMode = function (mode) {
        this.drawingSettings.currentMode = mode;
        this.resetCanvas();
        switch (mode) {
            case this.modes.PENCIL:
                this.draw();
                break;
            case this.modes.LINE:
                this.createLine();
                break;
            case this.modes.RECTANGLE:
                this.createRect();
                break;
            case this.modes.ELLIPSE:
                this.createEllipse();
                break;
            case this.modes.TRIANGLE:
                this.createTriangle();
                break;
            case this.modes.ERASER:
                this.eraserOn();
                break;
            case this.modes.ERASERDRAW:
                this.eraserDrawOn();
                break;
            case this.modes.SELECT:
                this.onSelectMode();
                break;
            case this.modes.MOVE:
                this.onMoveMode();
                break;
            case this.modes.TEXT:
                this.createText();
                break;
            default:
                this.draw();
        }
    };
    Board.prototype.resetCanvas = function () {
        var canvas = this.canvas;
        this.removeCanvasListener(canvas);
        canvas.selection = false;
        canvas.isDrawingMode = false;
        canvas.defaultCursor = 'auto';
        canvas.hoverCursor = 'auto';
        canvas.getObjects().map(function (item) { return item.set({ selectable: false }); });
        if (this.editedTextObject) {
            this.editedTextObject.exitEditing();
            this.editedTextObject = null;
        }
    };
    Board.prototype.throttle = function (f, delay) {
        if (delay === void 0) { delay = 300; }
        var timer = 0;
        return function () {
            var _this = this;
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            clearTimeout(timer);
            timer = setTimeout(function () { return f.apply(_this, args); }, delay);
        };
    };
    Board.prototype.handleResize = function (callback) {
        var resize_ob = new ResizeObserver(this.throttle(callback, 300));
        return resize_ob;
    };
    Board.prototype.resizeCanvas = function (canvas, whiteboard) {
        return function () {
            var width = whiteboard.clientWidth - 110;
            var height = whiteboard.clientHeight - 110;
            this.changeZoom({ scale: 1 });
            // const scale = width / canvas.getWidth();
            // const zoom = canvas.getZoom() * scale;
            canvas.setDimensions({ width: width, height: height });
            // canvas.setViewportTransform([zoom, 0, 0, zoom, 0, 0]);
        };
    };
    Board.prototype.removeCanvasListener = function () {
        var canvas = this.canvas;
        canvas.off('mouse:down');
        canvas.off('mouse:move');
        canvas.off('mouse:up');
        canvas.off('mouse:over');
    };
    Board.prototype.draw = function () {
        var canvas = this.canvas;
        var drawingSettings = this.drawingSettings;
        canvas.freeDrawingBrush = new fabric.PencilBrush(canvas);
        canvas.freeDrawingBrush.width = drawingSettings.brushWidth;
        canvas.freeDrawingBrush.color = drawingSettings.currentColor;
        canvas.isDrawingMode = true;
        canvas.freeDrawingCursor = this.cursorPencil;
    };
    Board.prototype.eraserDrawOn = function () {
        var canvas = this.canvas;
        var drawingSettings = this.drawingSettings;
        canvas.freeDrawingBrush = new fabric.EraserBrush(canvas);
        canvas.freeDrawingBrush.width = drawingSettings.brushWidth;
        canvas.freeDrawingBrush.color = drawingSettings.currentColor;
        canvas.isDrawingMode = true;
        canvas.freeDrawingCursor = getCursor('eraser');
        canvas.defaultCursor = getCursor('eraser');
        // canvas.hoverCursor = getCursor('eraser');
    };
    Board.prototype.onMoveMode = function () {
        var canvas = this.canvas;
        canvas.isDrawingMode = false;
        var drawingSettings = this.drawingSettings;
        canvas.on('mouse:down', function (opt) {
            var evt = opt.e;
            // if (evt.altKey === true) {
            this.isDragging = true;
            this.lastPosX = evt.clientX;
            this.lastPosY = evt.clientY;
            canvas.defaultCursor = 'all-scroll';
            canvas.hoverCursor = 'all-scroll';
            // }
        });
        // 移动鼠标事件
        canvas.on('mouse:move', function (opt) {
            if (this.isDragging) {
                var e = opt.e;
                var vpt = this.viewportTransform;
                vpt[4] += e.clientX - this.lastPosX;
                vpt[5] += e.clientY - this.lastPosY;
                this.requestRenderAll();
                this.lastPosX = e.clientX;
                this.lastPosY = e.clientY;
            }
        });
        // 松开鼠标事件
        canvas.on('mouse:up', function (opt) {
            this.setViewportTransform(this.viewportTransform);
            this.isDragging = false;
            canvas.defaultCursor = 'auto';
            canvas.hoverCursor = 'all-scroll';
        });
    };
    Board.prototype.clearCanvas = function () {
        var canvas = this.canvas;
        canvas.getObjects().forEach(function (item) {
            if (item !== canvas.backgroundImage) {
                canvas.remove(item);
            }
        });
        canvas.setBackgroundImage(null, canvas.renderAll.bind(canvas));
    };
    Board.prototype.changeZoom = function (_a) {
        var point = _a.point, scale = _a.scale;
        if (!point) {
            var width = this.canvas.width;
            var height = this.canvas.height;
            point = { x: width / 2, y: height / 2 };
        }
        var minZoom = this.canvasConfig.minZoom;
        var maxZoom = this.canvasConfig.maxZoom;
        scale = this.canvas.getZoom() * scale;
        scale = scale < minZoom ? minZoom : scale > maxZoom ? maxZoom : scale;
        this.canvas.zoomToPoint(point, scale);
        this.onZoom({ point: point, scale: scale });
    };
    Board.prototype.resetZoom = function () {
        var vpt = this.canvas.viewportTransform;
        vpt[4] = 0;
        vpt[5] = 0;
        var width = this.canvas.width;
        var height = this.canvas.height;
        var point = { x: width / 2, y: height / 2 };
        var scale = 1;
        this.canvas.zoomToPoint(point, scale);
        this.onZoom({ point: point, scale: scale });
    };
    Board.prototype.onZoom = function (params) {
        this.addZoomListeners();
        this.canvas.fire('zoom:change', params);
    };
    Board.prototype.openPage = function (page) {
        var canvas = this.canvas;
        var center = canvas.getCenter();
        fabric.Image.fromURL(page, function (img) {
            if (img.width > img.height) {
                img.scaleToWidth(canvas.width);
            }
            else {
                img.scaleToHeight(canvas.height - 100);
            }
            canvas.setBackgroundImage(img, canvas.renderAll.bind(canvas), {
                top: center.top,
                left: center.left,
                originX: 'center',
                originY: 'center',
            });
            canvas.renderAll();
        });
    };
    Board.prototype.getCanvasContentBoundaries = function () {
        var canvas = this.canvas;
        var objects = canvas.getObjects();
        // Initialize variables for min and max coordinates
        var minX = 10000;
        var minY = 10000;
        var maxX = -10000;
        var maxY = -10000;
        // Iterate through objects to find minimum and maximum coordinates
        objects.forEach(function (object) {
            var objectBoundingBox = object.getBoundingRect();
            minX = Math.min(minX, objectBoundingBox.left);
            minY = Math.min(minY, objectBoundingBox.top);
            maxX = Math.max(maxX, objectBoundingBox.left + objectBoundingBox.width);
            maxY = Math.max(maxY, objectBoundingBox.top + objectBoundingBox.height);
        });
        // Calculate canvas size based on content
        var width = maxX - minX;
        var height = maxY - minY;
        return { minX: minX, minY: minY, maxX: maxX, maxY: maxY, width: width, height: height };
    };
    Board.prototype.removeBoard = function () {
        this.element.disconnect();
        if (this.canvas) {
            this.canvas.off();
            this.canvas.dispose();
        }
        this.canvas = null;
    };
    return Board;
}());
export { Board };
//# sourceMappingURL=Board.Class.js.map